@@ -9,41 +9,37 @@
 # Contact: ops@keepersecurity.com
 #
 
-import itertools
-import json
 import base64
 import collections
-import re
-import getpass
-import google
-import time
-import os
 import hashlib
+import json
 import logging
-import math
-import urllib.parse
-from typing import Optional, Tuple, Iterable, List
-
+import os
+import re
 from datetime import datetime
+from typing import Optional, Tuple, Iterable, List, Dict, Any
 
-from . import constants, rest_api, loginv3, utils, crypto
-from .proto import client_pb2 as client_proto, APIRequest_pb2 as proto, record_pb2 as records
-from .subfolder import BaseFolderNode, UserFolderNode, SharedFolderNode, SharedFolderFolderNode, RootFolderNode
-from .record import Record
-from .shared_folder import SharedFolder
-from .team import Team
-from .error import AuthenticationError, CommunicationError, CryptoError, KeeperApiError
-from .params import KeeperParams, LAST_RECORD_UID
-from .display import bcolors
-from .recordv3 import RecordV3
-from .ttk import TTK
-
+import google
+import itertools
+import math
+import time
 from Cryptodome import Random
+from Cryptodome.Cipher import AES, PKCS1_v1_5
 from Cryptodome.Hash import SHA256
 from Cryptodome.PublicKey import RSA
-from Cryptodome.Cipher import AES, PKCS1_v1_5
 
+from . import constants, rest_api, loginv3, utils, crypto
+from .display import bcolors
 from .enterprise import query_enterprise as qe
+from .error import CryptoError, KeeperApiError
+from .params import KeeperParams, LAST_RECORD_UID
+from .proto import client_pb2 as client_proto, APIRequest_pb2 as proto, record_pb2 as records
+from .record import Record
+from .recordv3 import RecordV3
+from .shared_folder import SharedFolder
+from .subfolder import BaseFolderNode, UserFolderNode, SharedFolderNode, SharedFolderFolderNode, RootFolderNode
+from .team import Team
+from .ttk import TTK
 
 current_milli_time = lambda: int(round(time.time() * 1000))
 
@@ -233,7 +229,7 @@ NON_SHARED_DATA_SCOPE = ['non_shared_data']
 EXPLICIT = ['explicit']
 
 
-def sync_down(params):
+def sync_down(params, record_types=False):
     """Sync full or partial data down to the client"""
 
     params.sync_data = False
@@ -241,10 +237,15 @@ def sync_down(params):
     if params.revision == 0:
         logging.info('Syncing...')
 
+    includes = FOLDER_SCOPE + ['user_auth']
+    skip_records = params.config and 'skip_records' in params.config and params.config['skip_records'] is True
+    if not skip_records:
+        includes += RECORD_SCOPE
+
     rq = {
         'command': 'sync_down',
         'revision': params.revision or 0,
-        'include': FOLDER_SCOPE + RECORD_SCOPE + ['user_auth'] + EXPLICIT
+        'include': includes + EXPLICIT
     }
     response_json = communicate(params, rq)
 
@@ -766,17 +767,7 @@ def sync_down(params):
             except Exception as e:
                 logging.debug('Decrypt bw data: %s', e)
 
-    if 'full_sync' in response_json:
-        if params.breach_watch:
-            weak_count = 0
-            for _ in params.breach_watch.get_records_by_status(params, ['WEAK', 'BREACHED']):
-                weak_count += 1
-            if weak_count > 0:
-                logging.info(bcolors.WARNING +
-                             f'The number of records that are affected by breaches or contain high-risk passwords: {weak_count}' +
-                             '\nUse \"breachwatch list\" command to get more details' +
-                             bcolors.ENDC)
-
+    if 'full_sync' in response_json or record_types:
         # Record V3 types cache population
         v3_enabled = params.settings.get('record_types_enabled') if params.settings and isinstance(params.settings.get('record_types_enabled'), bool) else False
         if v3_enabled:
@@ -788,8 +779,28 @@ def sync_down(params):
 
             if len(record_types_rs.recordTypes) > 0:
                 params.record_type_cache = {}
+                conflict_type_id = 1000000
                 for rt in record_types_rs.recordTypes:
-                    params.record_type_cache[rt.recordTypeId] = rt.content
+                    type_id = rt.recordTypeId
+                    if rt.scope == records.RT_ENTERPRISE:
+                        type_id += 1000
+                    elif rt.scope == records.RT_USER:
+                        continue
+                    while type_id in params.record_type_cache:
+                        conflict_type_id += 1
+                        type_id = conflict_type_id
+                    params.record_type_cache[type_id] = rt.content
+
+    if 'full_sync' in response_json:
+        if params.breach_watch:
+            weak_count = 0
+            for _ in params.breach_watch.get_records_by_status(params, ['WEAK', 'BREACHED']):
+                weak_count += 1
+            if weak_count > 0:
+                logging.info(bcolors.WARNING +
+                             f'The number of records that are affected by breaches or contain high-risk passwords: {weak_count}' +
+                             '\nUse \"breachwatch list\" command to get more details' +
+                             bcolors.ENDC)
 
         record_count = 0
         valid_versions = {2, 3}
@@ -798,6 +809,9 @@ def sync_down(params):
                 record_count += 1
         if record_count:
             logging.info('Decrypted [%d] record(s)', record_count)
+        if skip_records:
+            logging.warning(bcolors.FAIL + 'Record loading is prevented in the configuration file.' + bcolors.ENDC)
+            logging.info('To load records use \"sync-down --force\" command.')
 
 
 def create_auth_verifier(password, salt, iterations):
@@ -1492,8 +1506,8 @@ def get_pb2_record_update(params, rec, **kwargs):
     return record_rq
 
 
-def get_record_v3_response(params, rq, endpoint, record_rq_by_uid, silent=False):
-    # type: (KeeperParams, records._message.Message, str, dict[dict]) -> Optional[bool]
+def get_record_v3_response(params, rq, endpoint, record_rq_by_uid, silent=True):
+    # type: (KeeperParams, Any, str, dict[dict], bool) -> Optional[bool]
     rs = communicate_rest(params, rq, endpoint)
     records_modify_rs = records.RecordsModifyResponse()
     records_modify_rs.ParseFromString(rs)
@@ -1528,7 +1542,7 @@ def get_record_v3_response(params, rq, endpoint, record_rq_by_uid, silent=False)
 
         record_rq['revision'] = new_revision
 
-    sync_down(params)
+    params.sync_data = True
     return True
 
 
@@ -1627,7 +1641,6 @@ def add_record_v3(params, record, **kwargs):   # type: (KeeperParams, dict, ...)
         Takes a Record() object, converts to record JSON
         and pushes to the Keeper cloud API
     """
-    from .commands.recordv3 import RecordTypeInfo
 
     record_rq = record
     if record_rq is None:
@@ -1819,7 +1832,7 @@ def prepare_folder_tree(params):
 
 
 def resolve_record_permission_path(params, record_uid, permission):
-    # type: (KeeperParams, str, str) -> dict or None
+    # type: (KeeperParams, str, str) -> Optional[Dict]
 
     for ap in enumerate_record_access_paths(params, record_uid):
         if ap.get(permission):
