@@ -14,15 +14,14 @@ import argparse
 import base64
 import copy
 import datetime
+import operator
 import time
 import json
 import gzip
 import logging
-import os
 import platform
 import re
-import sqlite3
-from typing import Optional, List
+from typing import Optional, List, Union, Tuple
 
 import requests
 import socket
@@ -37,12 +36,16 @@ from .recordv2 import RecordAddCommand
 from .helpers import audit_report
 from .enterprise_common import EnterpriseCommand
 from .base import user_choice, suppress_exit, raise_parse_exception, dump_report_data, Command
-from .. import api, utils, crypto
+from .. import api
 from ..error import CommandError
 from ..params import KeeperParams
 from ..proto import enterprise_pb2
 from .register import EMAIL_PATTERN
-
+from ..sox import sox_data
+from ..sox.sox_data import RebuildTask
+from ..sox.sox_types import RecordPermissions
+from ..sox.storage_types import StorageRecordAging
+from typing import Dict, Callable
 
 audit_report_parser = argparse.ArgumentParser(prog='audit-report', description='Run an audit trail report.')
 audit_report_parser.add_argument('--syntax-help', dest='syntax_help', action='store_true', help='display help')
@@ -61,7 +64,7 @@ audit_report_parser.add_argument('--aggregate', dest='aggregate', action='append
 audit_report_parser.add_argument('--timezone', dest='timezone', action='store',
                                  help='return results for specific timezone')
 audit_report_parser.add_argument('--limit', dest='limit', type=int, action='store',
-                                 help='maximum number of returned rows')
+                                 help='maximum number of returned rows (set to -1 to get all rows for raw report-type)')
 audit_report_parser.add_argument('--order', dest='order', action='store', choices=['desc', 'asc'],
                                  help='sort order')
 audit_report_parser.add_argument('--created', dest='created', action='store',
@@ -112,9 +115,40 @@ aging_report_parser.add_argument('--username', dest='username', action='store',
 aging_report_parser.error = raise_parse_exception
 aging_report_parser.exit = suppress_exit
 
+action_report_parser = argparse.ArgumentParser(prog='action-report', description='Run a user action report.')
+action_report_parser.add_argument('--target', '-t', dest='target_user_status', action='store',
+                                  choices=['no-logon', 'no-update', 'locked'], default='no-logon',
+                                  help='user status to report on')
+action_report_parser.add_argument('--days-since', '-d', dest='days_since', action='store', type=int,
+                                  help='number of days since event of interest (e.g., login, record add/update, lock)')
+action_report_parser.add_argument('--output', dest='output', action='store', help='path to resulting output file')
+action_report_parser.add_argument('--format', dest='format', action='store', choices=['table', 'csv', 'json'],
+                                  default='table', help='format of output')
+action_report_parser.add_argument('--apply-action', '-a', dest='apply_action', action='store',
+                                  choices=['lock', 'delete', 'none'], default='none',
+                                  help='admin action to apply to each user in the report')
+action_report_parser.add_argument('--dry-run', '-n', dest='dry_run', default=False, action='store_true',
+                                  help='flag to enable dry-run mode')
+
+compliance_report_parser = argparse.ArgumentParser(prog='compliance-report', description='Run a SOX compliance report.')
+compliance_report_parser.add_argument('--rebuild', '-r', action='store_true', help='Rebuild local data from source')
+compliance_report_parser.add_argument('--node', action='store', help='ID or name of node (defaults to root node)')
+compliance_report_parser.add_argument('--username', '-u', action='append',
+                                  help='user(s) to include in report (set option once for each user to include)')
+compliance_report_parser.add_argument('--job-title', '-jt', action='append',
+                                  help='job titles to include in report (set option once for each title to include)')
+compliance_report_parser.add_argument('--shared', action='store_true',
+                                      help='flag for excluding non-shared records from report')
+compliance_report_parser.add_argument('--output', dest='output', action='store',
+                                  help='path to resulting output file (if any)')
+compliance_report_parser.add_argument('--format', dest='format', action='store', choices=['table', 'csv', 'json'],
+                                  default='table', help='format of output')
 
 syslog_templates = None  # type: Optional[List[str]]
 
+API_EVENT_SUMMARY_ROW_LIMIT = 2000
+API_EVENT_RAW_ROW_LIMIT = 1000
+
 
 def load_syslog_templates(params):
     global syslog_templates
@@ -1105,10 +1139,9 @@ class AuditReportCommand(Command):
                 aggregates = kwargs['aggregate']
                 rq['aggregate'] = aggregates
 
-        if 'limit' in kwargs and kwargs['limit']:
-            rq['limit'] = kwargs['limit']
-        else:
-            rq['limit'] = 50
+        user_limit = kwargs.get('limit')
+        rq_limit = 50 if user_limit is None else user_limit if user_limit > 0 else API_EVENT_RAW_ROW_LIMIT
+        rq['limit'] = min(rq_limit, API_EVENT_RAW_ROW_LIMIT)
 
         if kwargs.get('order'):
             rq['order'] = 'ascending' if kwargs['order'] == 'asc' else 'descending'
@@ -1187,29 +1220,58 @@ class AuditReportCommand(Command):
         if report_type == 'raw':
             fields.extend(audit_report.RAW_FIELDS)
             misc_fields = list(audit_report.MISC_FIELDS) if kwargs.get('report_format') == 'fields' else ['message']
-
-            for event in rs['audit_event_overview_report_rows']:
-                if misc_fields:
-                    lenf = len(fields)
-                    for mf in misc_fields:
-                        if mf == 'message':
-                            fields.append(mf)
-                        elif mf in event:
-                            val = event.get(mf)
-                            if val:
+            incomplete = True
+            while incomplete:
+                events = rs.get('audit_event_overview_report_rows')
+                for event in events:
+                    if misc_fields:
+                        lenf = len(fields)
+                        for mf in misc_fields:
+                            if mf == 'message':
                                 fields.append(mf)
-                                if mf in audit_report.lookup_types:
-                                    fields.extend(audit_report.lookup_types[mf].fields)
-                    if len(fields) > lenf:
-                        for f in fields[lenf:]:
-                            if f not in audit_report.fields_to_uid_name:
-                                misc_fields.remove(f)
+                            elif mf in event:
+                                val = event.get(mf)
+                                if val:
+                                    fields.append(mf)
+                                    if mf in audit_report.lookup_types:
+                                        fields.extend(audit_report.lookup_types[mf].fields)
+                        if len(fields) > lenf:
+                            for f in fields[lenf:]:
+                                if f not in audit_report.fields_to_uid_name:
+                                    misc_fields.remove(f)
 
-                row = []
-                for field in fields:
-                    value = self.get_value(params, field, event)
-                    row.append(self.convert_value(field, value, details=details, params=params))
-                table.append(row)
+                    row = []
+                    for field in fields:
+                        value = self.get_value(params, field, event)
+                        row.append(self.convert_value(field, value, details=details, params=params))
+                    table.append(row)
+                incomplete = len(events) >= API_EVENT_RAW_ROW_LIMIT
+                if incomplete:
+                    asc = rq.get('order') == 'ascending'
+                    first_key, last_key = ('min', 'max') if asc else ('max', 'min')
+                    rq_filter = rq.get('filter', {})
+                    rq_period = rq_filter.get('created', {})
+                    period = {first_key: int(events[-1]['created'])}
+                    if not isinstance(rq_period, dict) or rq_period.get(last_key) is None:
+                        last_rq = {**rq}
+                        reverse = 'descending' if asc else 'ascending'
+                        last_rq['order'] = reverse
+                        last_rq['limit'] = 1
+                        rs = api.communicate(params, last_rq)
+                        last_row = rs.get('audit_event_overview_report_rows')[0]
+                        period[last_key] = int(last_row['created'])
+                    else:
+                        period[last_key] = rq_period.get(last_key)
+                    rq_filter['created'] = period
+                    rq['filter'] = rq_filter
+                    if user_limit and user_limit >= API_EVENT_RAW_ROW_LIMIT:
+                        missing = user_limit - len(table)
+                        if missing < API_EVENT_RAW_ROW_LIMIT:
+                            if missing > 0:
+                                rq['limit'] = missing
+                            else:
+                                break
+                    rs = api.communicate(params, rq)
             return dump_report_data(table, fields, fmt=kwargs.get('format'), filename=kwargs.get('output'))
         else:
             if aggregates:
@@ -1301,218 +1363,358 @@ class AgingReportCommand(Command):
         return aging_report_parser
 
     def execute(self, params, **kwargs):
+        def get_floor(period):  # type: (str) -> Union[datetime.datetime, None]
+            dt = datetime.datetime.now()
+            if not period:
+                logging.info('\n\nThe default password aging period is 3 months\n'
+                             'To change this value pass --period=[PERIOD] parameter\n'
+                             '[PERIOD] example: 10d for 10 days; 3m for 3 months; 1y for 1 year\n\n')
+                period = '3m'
+            co = period[-1]
+            try:
+                va = abs(int(period[:-1]))
+            except:
+                logging.warning(f'Invalid period: {period}')
+                return None
+
+            if co != 'd':
+                if co == 'm':
+                    va *= 30
+                elif co == 'y':
+                    va *= 365
+                else:
+                    logging.warning(f'Invalid period: {period}')
+                    return None
+            return dt - datetime.timedelta(days=va)
+
         enterprise_id = next(((x['node_id'] >> 32) for x in params.enterprise['nodes']), 0)
-        path = os.path.dirname(os.path.abspath(params.config_filename or '1'))
-        database_name = os.path.join(path, f'sox_data_{enterprise_id}.db')
-
-        if kwargs.get('rebuild') and os.path.exists(database_name):
-            os.remove(database_name)
-
-        if not AgingReportCommand.data_updated:
-            AgingReportCommand.prepare_database(params, database_name)
-
-        dt = datetime.datetime.now()
-        period = kwargs.get('period')
-        if not period:
-            logging.info('\n\nThe default password aging period is 3 months\n'
-                  'To change this value pass --period=[PERIOD] parameter\n'
-                  '[PERIOD] example: 10d for 10 days; 3m for 3 months; 1y for 1 year\n\n')
-            period = '3m'
-
-        co = period[-1]
-        try:
-            va = abs(int(period[:-1]))
-        except:
-            logging.warning(f'Invalid period: {period}')
+        dt = get_floor(kwargs.get('period'))
+        if dt is None:
             return
+        period_min_ts = int(dt.timestamp())
 
-        if co == 'd':
-            dt = dt - datetime.timedelta(days=-va)
-        elif co == 'm':
-            month = dt.month
-            year = dt.year
-            month -= va
-            while month < 1:
-                month += 12
-                year -= 1
-            dt = dt.replace(month=month, year=year)
-        elif co == 'y':
-            dt = dt.replace(year=dt.year-va)
+        from ..sox import get_prelim_data
+        sd = get_prelim_data(params, enterprise_id, rebuild=kwargs.get('rebuild'), min_updated=period_min_ts)
+        sd = self.update_aging_data(params, sd, period_start_ts=period_min_ts)
+
+        users = {uid: user for uid, user in sd.get_users().items() if user.status == enterprise_pb2.OK}
+        uid_lookup = {user.email: uid for uid, user in users.items()}
+        username = kwargs.get('username')
+        if username and username not in uid_lookup:
+            logging.info(f'user {username} is not a valid enterprise user')
+            return
         else:
-            logging.warning(f'Invalid period: {period}')
-            return
-
-        user_lookup = {x['enterprise_user_id']: x['username'] for x in params.enterprise['users']}
-        enterprise_user_id = 0
-        user = kwargs.get('username')
-        if user:
-            user = user.lower()
-            enterprise_user_id = next((x[0] for x in user_lookup.items() if x[1].lower() == user), 0)
-
-        tree_key = params.enterprise['unencrypted_tree_key']
-        ecc_key = utils.base64_url_decode(params.enterprise['keys']['ecc_encrypted_private_key'])
-        ecc_key = crypto.decrypt_aes_v2(ecc_key, tree_key)
-        ec_private_key = crypto.load_ec_private_key(ecc_key)
-
+            user_uids = [uid_lookup.get(username)] if username is not None else None
         output_format = kwargs.get('format', 'table')
         date_ts = int(dt.timestamp())
-        error_count = 0
         columns = ['owner', 'title', 'password_changed', 'shared', 'record_url'] if output_format == 'json' else \
             ['Owner', 'Record Title', 'Last Password Change', 'Shared', 'Record URL']
         table = []
-        with sqlite3.connect(os.path.join(os.path.dirname(__file__), database_name)) as connection:
-            cursor = connection.execute('select record_uid, user, created, modified, shared, encrypted_data from audit_data')
-
-            for record_uid, user_id, created, modified, shared, encrypted_data in cursor:
-                if user_id not in user_lookup:
-                    continue
-                if enterprise_user_id > 0 and user_id != enterprise_user_id:
-                    continue
-                if created or modified:
-                    if modified:
-                        if modified >= date_ts:
-                            continue
-                    else:
-                        if created >= date_ts:
-                            continue
-                try:
-                    audit_json = crypto.decrypt_ec(utils.base64_url_decode(encrypted_data), ec_private_key)
-                    audit = json.loads(audit_json.decode())
-                    row = [user_lookup[user_id], audit.get('title', record_uid),
-                           datetime.datetime.fromtimestamp(modified) if modified else None,
-                           True if shared > 0 else False,
-                           f'https://{params.server}/vault/#detail/{record_uid}']
-                    table.append(row)
-                except:
-                    error_count += 1
-
+        user_records = sd.get_user_records(user_uids)
+        for ur in user_records:
+            created_ts = ur.record.created
+            change_ts = ur.record.last_pw_change
+            created_after_date = created_ts and (created_ts >= date_ts)
+            pw_changed_after_date = change_ts and (change_ts >= date_ts)
+            if created_after_date or pw_changed_after_date:
+                continue
+            else:
+                email = sd.get_user(ur.user_uid).email
+                ts = change_ts or created_ts
+                change_dt = datetime.datetime.fromtimestamp(ts) if ts else None
+                record_url = f'https://{params.server}/value/#detail/{ur.record.record_uid}'
+                row = [email, ur.record.data.get('title'), change_dt, ur.record.shared, record_url]
+                table.append(row)
         return dump_report_data(table, columns, fmt=output_format, filename=kwargs.get('output'))
 
     @staticmethod
     def get_database_path(params):
         pass
 
-    @staticmethod
-    def prepare_database(params, database_name):
-        user_lookup = {x['username']: x['enterprise_user_id'] for x in params.enterprise['users']}
-        last_audit_time = 0
-        report_command = AuditReportCommand()
-        from_date = datetime.datetime.now()
-        from_date = from_date.replace(year=(from_date.year - 5))
+    def update_aging_data(self, params, sox, period_start_ts):
+        # type: (KeeperParams, sox_data.SoxData, int) -> sox_data.SoxData
+        def get_record_event_dates(params, record_uids, event_type, ts_floor):
+            # type: (KeeperParams, List[Union[str, int]], str, int) -> Dict[str, int]
+            rq = {
+                'command':      'get_audit_event_reports',
+                'scope':        'enterprise',
+                'report_type':  'span',
+                'event_type':   event_type,
+                'columns':      ['record_uid'],
+                'aggregate':    ['last_created'],
+                'limit':        API_EVENT_SUMMARY_ROW_LIMIT
+            }
+            ts_lookup = dict()
+            while record_uids:
+                chunk = record_uids[:API_EVENT_SUMMARY_ROW_LIMIT]
+                record_uids = record_uids[API_EVENT_SUMMARY_ROW_LIMIT:]
+                rq_filter = {'record_uid': chunk, 'created': {'min': ts_floor}}
+                rq['filter'] = rq_filter
+                rs = api.communicate(params, rq)
+                events = rs['audit_event_overview_report_rows']
+                chunk_ts_lookup = {event['record_uid']: int(event['last_created']) for event in events}
+                ts_lookup.update(chunk_ts_lookup)
+            return ts_lookup
 
-        # get SOX data
-        with sqlite3.connect(database_name) as connection:
-            rs = connection.execute('pragma table_info(\'audit_data\')').fetchall()
-            if rs:
-                rs = connection.execute('select max(modified) from audit_data where modified is not null').fetchall()
-                try:
-                    if rs[0][0] is not None:
-                        last_audit_time = rs[0][0]
-                except:
-                    pass
+        def update_record_aging(params, sd, r_uids, event_type, set_ent_field_fn, on_updated, msg, ts_floor=0):
+            # type: (KeeperParams, sox_data.SoxData, List[int], str, Callable, Callable, str, int) -> None
+            logging.info(msg)
+            ts_lookup = get_record_event_dates(params, r_uids, event_type, ts_floor)
+            updated_entities = []
+            for rec_uid in ts_lookup:
+                entity = sd.storage.record_aging.get_entity(rec_uid) or StorageRecordAging(rec_uid)
+                entity = set_ent_field_fn(entity, ts_lookup.get(rec_uid))
+                updated_entities.append(entity)
+            rebuild_task = RebuildTask(False)
+            rebuild_task.update_records(ts_lookup.keys())
+            sd.storage.record_aging.put_entities(updated_entities)
+            sd.rebuild_data(rebuild_task)
+            on_updated()
 
-            else:
-                connection.execute('create table audit_data (record_uid TEXT NOT NULL, user INTEGER NOT NULL, encrypted_data TEXT NOT NULL, '
-                                   'shared INTEGER NOT NULL, created INTEGER NULL, modified INTEGER NULL, primary key (record_uid))')
-                connection.execute('create index audit_data_user_idx on audit_data(user)')
+        def update_record_created_times(params, sd, ts_floor=0):   # type: (KeeperParams, sox_data.SoxData, int) -> None
+            def update_ent(ent, val):   # type: (StorageRecordAging, int) -> StorageRecordAging
+                ent.created = val
+                return ent
 
-                record_uids = set()
-                logging.info('Loading record information.')
-                user_ids = list(user_lookup.values())
-                loaded = 0
-                while user_ids:
-                    continuation_token = b''
-                    chunk = user_ids[:999]
-                    user_ids = user_ids[999:]
-                    rq = enterprise_pb2.PreliminaryComplianceDataRequest()
-                    rq.enterpriseUserIds.extend(chunk)
-                    rq.includeNonShared = True
-                    has_more = True
-                    total_records = 0
-                    while has_more:
-                        if continuation_token:
-                            rq.continuationToken = continuation_token
-                        rs = api.communicate_rest(params, rq, 'enterprise/get_preliminary_compliance_data',
-                                                  rs_type=enterprise_pb2.PreliminaryComplianceDataResponse)
-                        has_more = rs.hasMore
-                        continuation_token = rs.continuationToken
-                        if rs.totalMatchingRecords:
-                            total_records = rs.totalMatchingRecords
-                        for user in rs.auditUserData:
-                            loaded += len(user.auditUserRecords)
+            on_update_fn = sd.storage.set_records_dated
+            r_uids = [uid for uid, record in sd.get_records().items() if not record.created]
+            event_type = 'record_add'
+            msg = 'Loading record creation time...'
+            update_record_aging(params, sd, r_uids, event_type, update_ent, on_update_fn, msg, ts_floor)
 
-                        def get_audit_data():
-                            for user in rs.auditUserData:
-                                if user.status == enterprise_pb2.OK:
-                                    for record in user.auditUserRecords:
-                                        record_uid = utils.base64_url_encode(record.recordUid)
-                                        record_uids.add(record_uid)
-                                        yield (record_uid, user.enterpriseUserId, 1 if record.shared else 0,
-                                               utils.base64_url_encode(record.encryptedData))
+        def update_pw_change_times(params, sd, ts_floor=0):
+            def update_ent(ent, val):   # type: (StorageRecordAging, int) -> StorageRecordAging
+                ent.last_pw_change = val
+                return ent
 
-                        connection.executemany('insert into audit_data (record_uid, user, shared, encrypted_data) values (?, ?, ?, ?) ' +
-                                               'on conflict (record_uid) do update set ' +
-                                               'user=excluded.user, shared=excluded.shared, encrypted_data=excluded.encrypted_data',
-                                               get_audit_data())
+            on_update_fn = sd.storage.set_last_pw_audit
+            event_type = 'record_password_change'
+            msg = 'Loading record password change information...'
+            records = {id: rec for id, rec in sd.get_records().items() if rec.created and rec.last_pw_change < ts_floor}
+            rec_uids = list(records.keys())
+            created_timestamps = {record.created for record in records.values()}
+            ts_floor = min(*created_timestamps, ts_floor) if created_timestamps else ts_floor
+            update_record_aging(params, sd, rec_uids, event_type, update_ent, on_update_fn, msg, ts_floor)
 
-                        connection.commit()
-                logging.info('Loading record creation time')
-                loaded = 0
-                has_more = True
-                last_created_time = from_date.timestamp()
-                while has_more:
-                    has_more = False
-                    report_data = report_command.execute(
-                        params, report_type='span', columns=['record_uid'], aggregate=['last_created'], format='json',
-                        order='asc', limit=1000, created=f'> {int(last_created_time)}', event_type='record_add')
-                    data = json.loads(report_data)
-                    if not data:
-                        break
-                    loaded += len(data)
-                    if len(data) == 1000:
-                        last_event = data[-1]
-                        created_date = datetime.datetime.strptime(last_event['last_created'], '%Y-%m-%dT%H:%M:%S%z')
-                        last_created_time = created_date.timestamp()
-                        has_more = True
+        min_dt = datetime.datetime.now() - datetime.timedelta(days=365) * 5
+        min_ts = int(min_dt.timestamp())
+        if sox.storage.records_dated < period_start_ts:
+            update_record_created_times(params, sox, min_ts)
+        if sox.storage.last_pw_audit < period_start_ts:
+            update_pw_change_times(params, sox, ts_floor=period_start_ts)
+        return sox
 
-                    def get_date_created():
-                        for event in data:
-                            last_created = event['last_created']
-                            dt = datetime.datetime.strptime(last_created, '%Y-%m-%dT%H:%M:%S%z')
-                            record_uid = event['record_uid']
-                            if record_uid in record_uids:
-                                yield int(dt.timestamp()), event['record_uid']
 
-                    connection.executemany('update or ignore audit_data set created = ? where record_uid = ?', get_date_created())
-                    connection.commit()
+class ActionReportCommand(EnterpriseCommand):
+    def get_parser(self):  # type: () -> Optional[argparse.ArgumentParser]
+        return action_report_parser
 
-            has_more = True
-            if last_audit_time == 0:
-                last_audit_time = from_date.timestamp()
-            loaded = 0
-            while has_more:
-                has_more = False
-                report_data = report_command.execute(params, report_type='span', columns=['record_uid'], aggregate=['last_created'], order='asc', limit=1000,
-                                                     created=f'> {int(last_audit_time)}', event_type='record_password_change', format='json')
-                data = json.loads(report_data)
-                if not data:
-                    break
-                if loaded == 0:
-                    logging.info('Loading record password change information.')
+    def execute(self, params, **kwargs):
+        def cmd_rq(cmd, **kwargs):
+            rq = {
+                'command': cmd,
+                'scope': 'enterprise',
+                **kwargs
+            }
+            return rq
 
-                loaded += len(data)
-                if len(data) == 1000:
-                    last_event = data[-1]
-                    modified_date = datetime.datetime.strptime(last_event['last_created'], '%Y-%m-%dT%H:%M:%S%z')
-                    last_audit_time = modified_date.timestamp()
-                    has_more = True
+        def report_rq(query_filter, cols=None):
+            rq = {
+                **cmd_rq('get_audit_event_reports'),
+                'report_type': 'span',
+                'columns': ['username'] if cols is None else cols,
+                'filter': query_filter,
+                'aggregate': ['last_created'],
+                'limit': API_EVENT_SUMMARY_ROW_LIMIT,
+            }
+            return rq
 
-                def get_password_data():
-                    for event in data:
-                        last_created = event['last_created']
-                        dt = datetime.datetime.strptime(last_created, '%Y-%m-%dT%H:%M:%S%z')
-                        yield int(dt.timestamp()), event['record_uid']
+        def get_excluded(candidates, query_filter, username_field='username'):
+            excluded = []
+            columns = [username_field]
+            get_events = len(candidates) > len(excluded)
+            while get_events:
+                rq = report_rq(query_filter, columns)
+                rs = api.communicate(params, rq)
+                events = rs['audit_event_overview_report_rows']
+                to_exclude = [event[username_field] for event in events]
+                excluded = excluded + to_exclude
+                get_events = len(events) >= API_EVENT_SUMMARY_ROW_LIMIT
+                if get_events:
+                    candidates = [user for user in candidates if user not in excluded]
+                    end = int(events[-1]['last_created'])
+                    query_filter['created']['max'] = end
+                    query_filter[username_field] = candidates
+            return excluded
 
-                connection.executemany('update or ignore audit_data set modified = ? where record_uid = ?',
-                                       get_password_data())
-                connection.commit()
+        def get_no_action_users(candidates, days_since, event_types, name_key='username'):
+            days_since = 30 if not isinstance(days_since, int) else days_since
+            now_dt = datetime.datetime.now()
+            min_dt = now_dt - datetime.timedelta(days=days_since)
+            start = int(min_dt.timestamp())
+            end = int(now_dt.timestamp())
+            period = {'min': start, 'max': end}
+            included = [candidate['username'] for candidate in candidates]
+            query_filter = {
+                'audit_event_type': ['login'] if event_types is None else event_types,
+                'created': period
+            }
+            excluded = get_excluded(included, query_filter, name_key)
+            return [user for user in candidates if user['username'] not in excluded]
+
+        def batch_apply_cmd(users, api_cmd_rq=None, dryrun=False, userid_field='enterprise_user_id'):
+            cmd_status = 'aborted' if api_cmd_rq else 'n/a'
+            affected = 0
+            server_msg = 'n/a'
+            cmd = None if api_cmd_rq is None else api_cmd_rq.get('command')
+            if api_cmd_rq is not None and len(users):
+                cmd_rqs = [{**api_cmd_rq, userid_field: user[userid_field]} for user in users]
+                rq = cmd_rq('execute', requests=cmd_rqs)
+                if dryrun:
+                    cmd_status = 'dry run'
+                else:
+                    rs = api.communicate(params, rq)
+                    cmd_status = rs.get('result')
+                    server_msg = rs.get('message')
+                    affected = len(users)
+                    if cmd_status == 'success':
+                        results = rs.get('results')
+                        fails = [result for result in results if result.get('result') != 'success']
+                        if any(fails):
+                            fail = fails[0]
+                            cmd_status = 'incomplete'
+                            affected = len(users) - len(fails)
+                            server_msg = fail.get('message')
+
+            return f'\tCOMMAND: {cmd}\n\tSTATUS: {cmd_status}\n\tSERVER MESSAGE: {server_msg}\n\tAFFECTED: {affected}'
+
+        def apply_admin_action(users, target_status='no-update', action='none', dryrun=False):
+            default_allowed = {'none'}
+            status_actions = {
+                'no-logon':     {*default_allowed, 'lock'},
+                'no-update':    {*default_allowed},
+                'locked':       {*default_allowed, 'delete'}
+            }
+            action_cmd_rqs = {
+                'none': None,
+                'lock': cmd_rq('enterprise_user_lock', lock='locked'),
+                'delete': cmd_rq('enterprise_user_delete')
+            }
+            invalid_action_msg = f'NONE (Action "{action}" not allowed on "{target_status}" users)'
+            is_valid_action = action in status_actions[target_status]
+            api_cmd_rq = action_cmd_rqs[action]
+            return batch_apply_cmd(users, api_cmd_rq, dryrun) if is_valid_action else invalid_action_msg
+
+        candidates = params.enterprise['users']
+        active = [user for user in candidates if user['status'] == 'active']
+        locked = [user for user in active if user['lock']]
+        target_status = kwargs['target_user_status']
+        days = kwargs['days_since']
+        if days is None:
+            days = 90 if target_status == 'locked' else 30
+
+        args_by_status = {
+            'no-logon': [active, days, ['login']],
+            'no-update': [active, days, ['record_add', 'record_update']],
+            'locked': [locked, days, ['lock_user'], 'to_username']
+        }
+        args = args_by_status[target_status]
+        target_users = get_no_action_users(*args)
+        usernames = [user['username'] for user in target_users]
+
+        admin_action = kwargs['apply_action']
+        dry_run = kwargs['dry_run']
+        action_msg = apply_admin_action(target_users, target_status, admin_action, dry_run)
+
+        title = f'Admin Action Taken:\n{action_msg}\n'
+        title += f'\n{len(usernames)} Users With "{target_status}" Status Older Than {days} Day(s): '
+        filepath = kwargs.get('output')
+        fmt = kwargs.get('format')
+        formatted = [[name] for name in usernames]
+
+        return dump_report_data(data=formatted, headers=['username'], title=title, fmt=fmt, filename=filepath)
+
+
+class ComplianceReportCommand(EnterpriseCommand):
+    def get_parser(self):  # type: () -> Optional[argparse.ArgumentParser]
+        return compliance_report_parser
+
+    def execute(self, params, **kwargs):
+        node_name_or_id = kwargs.get('node')
+        node_name_or_id = int(node_name_or_id) if node_name_or_id and node_name_or_id.isdecimal() else node_name_or_id
+        nodes = params.enterprise['nodes']
+        root_node_id = nodes[0].get('node_id', 0)
+        node_ids = (n.get('node_id') for n in nodes)
+        node_id_lookup = {n.get('data').get('displayname'): n.get('node_id') for n in nodes}
+        node_id = node_id_lookup.get(node_name_or_id) if node_name_or_id in node_id_lookup \
+            else node_name_or_id if node_name_or_id in node_ids \
+            else root_node_id
+        enterprise_id = node_id >> 32
+        max_data_age = datetime.timedelta(days=1)
+        min_data_ts = (datetime.datetime.now() - max_data_age).timestamp()
+
+        from ..sox import get_compliance_data
+        sd = get_compliance_data(params, node_id, enterprise_id, rebuild=kwargs.get('rebuild'), min_updated=min_data_ts)
+
+        report_fmt = kwargs.get('format', 'table')
+        headers = ['record_uid', 'username', 'permissions'] if report_fmt == 'json' \
+            else ['Record UID', 'Username', 'Permissions']
+        table = []
+
+        def filter_owners(owners):
+            names = kwargs.get('username')
+            filtered = [o for o in owners if o.email in names] if names else owners
+            job_titles = kwargs.get('job_title')
+            filtered = [o for o in filtered if o.job_title in job_titles] if job_titles else filtered
+            return [o for o in filtered if o.node_id == node_id] if node_id != root_node_id else filtered
+
+        def filter_records(records):
+            shared_only = kwargs.get('shared')
+            return [record for record in records if not shared_only or record.shared]
+
+        owners = filter_owners(sd.get_users().values())
+        shared_folders = sd.get_shared_folders()
+        permissions_lookup = dict()  # type: Dict[Tuple[str, str], str]
+
+        def update_permissions_lookup(plookup, r_uid, user, perm_bits):
+            lookup_key = r_uid, user.email
+            pbits = plookup.get(lookup_key, perm_bits)
+            plookup.update({lookup_key: perm_bits | pbits})
+
+        for owner in owners:
+            owner_records = filter_records(sd.get_records(owner.records).values())
+            for record in owner_records:
+                ruid = record.record_uid
+                for user_uid, permission_bits in record.user_permissions.items():
+                    user = sd.get_user(user_uid)
+                    update_permissions_lookup(permissions_lookup, ruid, user, permission_bits)
+                for folder in shared_folders.values():
+                    for rp in folder.record_permissions:
+                        if rp.record_uid == ruid:
+                            for user in sd.get_users(folder.users).values():
+                                update_permissions_lookup(permissions_lookup, ruid, user, rp.permission_bits)
+                            for team in sd.get_teams(folder.teams).values():
+                                for team_user in sd.get_users(team.users).values():
+                                    update_permissions_lookup(permissions_lookup, ruid, team_user, rp.permission_bits)
+
+        for key, permission_bits in permissions_lookup.items():
+            r_uid, email = key
+            table.append({'record_uid': r_uid, 'email': email, 'permissions': permission_bits})
+
+        def format_table(rows):
+            rows.sort(key=operator.itemgetter('permissions'), reverse=True)
+            rows.sort(key=operator.itemgetter('record_uid'))
+            last_ruid = ''
+            formatted_rows = []
+            for row in rows:
+                r_uid = row.get('record_uid')
+                formatted_ruid = r_uid if report_fmt != 'json' and r_uid != last_ruid else ''
+                permissions = RecordPermissions.to_permissions_str(row.get('permissions'))
+                formatted_rows.append([formatted_ruid, row.get('email'), permissions])
+                last_ruid = r_uid
+            return formatted_rows
+
+        table = format_table(table)
+        print(f'num rows in report = {len(table)}')
+        print(f'num unique records in report = {len([row for row in table if row[0]])}')
+        return dump_report_data(table, headers, fmt=report_fmt, filename=kwargs.get('output'))
