@@ -11,7 +11,6 @@
 
 import argparse
 import base64
-import datetime
 import json
 import logging
 import os
@@ -24,15 +23,15 @@ from Cryptodome.Cipher import AES
 from tabulate import tabulate
 
 from . import record_common
-from .base import user_choice, suppress_exit, raise_parse_exception, dump_report_data, Command
+from .base import user_choice, suppress_exit, raise_parse_exception, Command
 from .. import api, generator
 from .. import attachment
 from ..display import bcolors
 from ..error import CommandError
 from ..params import KeeperParams, LAST_RECORD_UID
-from ..proto.enterprise_pb2 import SharedRecordResponse
 from ..record import Record, get_totp_code
-from ..subfolder import BaseFolderNode, find_folders, try_resolve_path, get_folder_path, SharedFolderFolderNode, SharedFolderNode
+from ..subfolder import BaseFolderNode, find_folders, try_resolve_path, get_folder_path, SharedFolderFolderNode, \
+    SharedFolderNode
 
 
 def register_commands(commands):
@@ -45,9 +44,7 @@ def register_commands(commands):
     commands['upload-attachment'] = RecordUploadAttachmentCommand()
     commands['delete-attachment'] = RecordDeleteAttachmentCommand()
     commands['clipboard-copy'] = ClipboardCommand()
-    commands['record-history'] = RecordHistoryCommand()
     commands['totp'] = TotpCommand()
-    commands['shared-records-report'] = SharedRecordsReport()
 
 
 def register_command_info(aliases, command_info):
@@ -59,27 +56,12 @@ def register_command_info(aliases, command_info):
     aliases['cc'] = 'clipboard-copy'
     aliases['find-password'] = ('clipboard-copy', '--output=stdout')
     aliases['rh'] = 'record-history'
-    aliases['srr'] = 'shared-records-report'
 
-    for p in [get_info_parser, clipboard_copy_parser, record_history_parser, totp_parser,  add_parser, edit_parser, rm_parser,
-              append_parser, download_parser, upload_parser, delete_attachment_parser, shared_records_report_parser]:
+    for p in [get_info_parser, clipboard_copy_parser, totp_parser,  add_parser, edit_parser, rm_parser,
+              append_parser, download_parser, upload_parser, delete_attachment_parser]:
         command_info[p.prog] = p.description
 
 
-record_history_parser = argparse.ArgumentParser(prog='record-history|rh', description='Show the history of a record modifications.')
-record_history_parser.add_argument(
-    '-a', '--action',
-    dest='action',
-    choices=['list', 'diff', 'show', 'restore'],
-    action='store',
-    help="filter by record history type. (default: 'list'). --revision required with 'restore' action.",
-)
-record_history_parser.add_argument('-r', '--revision', dest='revision', type=int, action='store', help='only show the details for a specific revision')
-record_history_parser.add_argument('record', nargs='?', type=str, action='store', help='record path or UID')
-record_history_parser.error = raise_parse_exception
-record_history_parser.exit = suppress_exit
-
-
 totp_parser = argparse.ArgumentParser(prog='totp', description='Display the Two Factor Code for a record.')
 totp_parser.add_argument('-p', '--print', dest='print', action='store_true', help='print TOTP code to standard output')
 totp_parser.add_argument('record', nargs='?', type=str, action='store', help='record path or UID')
@@ -132,6 +114,7 @@ rm_parser.exit = suppress_exit
 
 get_info_parser = argparse.ArgumentParser(prog='get|g', description='Get the details of a record/folder/team by UID.')
 get_info_parser.add_argument('--format', dest='format', action='store', choices=['detail', 'json', 'password'], default='detail', help='output format.')
+get_info_parser.add_argument('--unmask', dest='unmask', action='store_true', help='display hidden field context')
 get_info_parser.add_argument('uid', type=str, action='store', help='UID')
 get_info_parser.error = raise_parse_exception
 get_info_parser.exit = suppress_exit
@@ -165,13 +148,6 @@ delete_attachment_parser.error = raise_parse_exception
 delete_attachment_parser.exit = suppress_exit
 
 
-shared_records_report_parser = argparse.ArgumentParser(prog='shared-records-report|srr', description='Report shared records for a logged-in user.')
-shared_records_report_parser.add_argument('--format', dest='format', choices=['json', 'csv', 'table'], default='table', help='Data format output')
-shared_records_report_parser.add_argument('name', type=str, nargs='?', help='file name')
-shared_records_report_parser.error = raise_parse_exception
-shared_records_report_parser.exit = suppress_exit
-
-
 class RecordUtils(object):
     parameter_pattern = re.compile(r'^\${([^:]+?):([^}]+?)}$')
 
@@ -664,6 +640,8 @@ class RecordGetUidCommand(Command):
                         ro['notes'] = r.notes
                     if r.custom_fields:
                         ro['custom_fields'] = r.custom_fields
+                    if r.totp:
+                        ro['totp'] = r.totp
                     if r.attachments:
                         ro['attachments'] = [{
                             'id': a.get('id'),
@@ -687,7 +665,7 @@ class RecordGetUidCommand(Command):
                 elif fmt == 'password':
                     print(r.password)
                 else:
-                    r.display(params=params)
+                    r.display(params=params, unmask=kwargs.get('unmask', False))
                 return
 
         if params.available_team_cache is None:
@@ -1056,278 +1034,6 @@ class ClipboardCommand(Command):
                 params.queue_audit_event('copy_password', record_uid=record_uid)
 
 
-class RecordHistoryCommand(Command):
-    def get_parser(self):
-        return record_history_parser
-
-    def execute(self, params, **kwargs):
-        record_name = kwargs['record'] if 'record' in kwargs else None
-        if not record_name:
-            self.get_parser().print_help()
-            return
-
-        record_uid = None
-        if record_name in params.record_cache:
-            record_uid = record_name
-        else:
-            rs = try_resolve_path(params, record_name)
-            if rs is not None:
-                folder, record_name = rs
-                if folder is not None and record_name is not None:
-                    folder_uid = folder.uid or ''
-                    if folder_uid in params.subfolder_record_cache:
-                        for uid in params.subfolder_record_cache[folder_uid]:
-                            r = api.get_record(params, uid)
-                            if r.title.lower() == record_name.lower():
-                                record_uid = uid
-                                break
-
-        if record_uid is None:
-            raise CommandError('history', 'Enter name or uid of existing record')
-
-        current_rec = params.record_cache[record_uid]
-        if record_uid in params.record_history:
-            _,revision = params.record_history[record_uid]
-            if revision < current_rec['revision']:
-                del params.record_history[record_uid]
-        if record_uid not in params.record_history:
-            rq = {
-                'command': 'get_record_history',
-                'record_uid': record_uid,
-                'client_time': api.current_milli_time
-            }
-            rs = api.communicate(params, rq)
-            params.record_history[record_uid] = (rs['history'], current_rec['revision'])
-
-        if record_uid in params.record_history:
-            action = kwargs.get('action') or 'list'
-            history,_ = params.record_history[record_uid]
-            history.sort(key=lambda x: x['revision'])
-            history.reverse()
-            length = len(history)
-            if length == 0:
-                logging.info('Record does not have history of edit')
-                return
-
-            if 'revision' in kwargs and kwargs['revision'] is not None:
-                revision = kwargs['revision']
-                if revision < 1 or revision > length+1:
-                    logging.error('Invalid revision %d: valid revisions 1..%d'.format(revision, length + 1))
-                    return
-                if not kwargs.get('action'):
-                    action = 'show'
-            else:
-                revision = 0
-
-            if action == 'list':
-                headers = ['Version', 'Modified By', 'Time Modified']
-                rows = []
-                for i, revision in enumerate(history):
-                    if 'client_modified_time' in revision:
-                        dt = datetime.datetime.fromtimestamp(revision['client_modified_time']/1000.0)
-                        tm = dt.strftime('%Y-%m-%d %H:%M:%S')
-                    else:
-                        tm = ''
-                    rows.append(['V.{}'.format(length-i), revision.get('user_name') or '', tm])
-                print(tabulate(rows, headers=headers))
-            elif action == 'show':
-                if revision == 0:
-                    revision = length + 1
-                current_rec = params.record_cache[record_uid]
-                key = current_rec['record_key_unencrypted']
-                rev = history[length - revision]
-                rec = RecordHistoryCommand.load_revision(params, key, rev)
-                print('\n{0:>20s}: V.{1}'.format('Revision', revision))
-                rec.display()
-            elif action == 'diff':
-                if revision == 0:
-                    revision = 1
-                current_rec = params.record_cache[record_uid]
-                key = current_rec['record_key_unencrypted']
-                rev = history[0]
-                record_next = RecordHistoryCommand.load_revision(params, key, rev)
-                rows = []
-                for current_revision in range(length, revision-1, -1):
-                    current_rev = history[length - current_revision]
-                    record_current = RecordHistoryCommand.load_revision(params, key, current_rev)
-                    added = False
-                    for d in RecordHistoryCommand.get_record_diffs(record_next, record_current):
-                        rows.append(['V.{}'.format(current_revision+1) if not added else '', d[0], d[1], d[2]])
-                        added = True
-                    record_next = record_current
-                record_current = Record()
-                added = False
-                for d in RecordHistoryCommand.get_record_diffs(record_next, record_current):
-                    rows.append(['V.{}'.format(1) if not added else '', d[0], d[1], d[2]])
-                    added = True
-                headers = ('Version', 'Field', 'New Value', 'Old Value')
-                print(tabulate(rows, headers=headers))
-            elif action == 'restore':
-                ro = api.resolve_record_write_path(params, record_uid)    # type: dict
-                if not ro:
-                    raise CommandError('history', 'You do not have permission to modify this record')
-                if revision == 0:
-                    raise CommandError('history', 'Invalid revision to restore: Revisions: 1-{0}'.format(length))
-
-                rev = history[length - revision]
-                if rev['version'] in {1,2}:
-                    current_rec = params.record_cache[record_uid]
-                    ro.update({
-                        'version': rev['version'],
-                        'client_modified_time': api.current_milli_time(),
-                        'revision': current_rec['revision'],
-                        'data': rev['data']
-                    })
-                    udata = current_rec.get('udata') or {}
-                    extra = {}
-                    if 'extra_unencrypted' in current_rec:
-                        extra = json.loads(current_rec['extra_unencrypted'].decode('utf-8'))
-                    if 'udata' in rev:
-                        udata.update(rev['udata'])
-                    udata['file_ids'] = []
-                    if 'files' in extra:
-                        del extra['files']
-
-                    key = current_rec['record_key_unencrypted']
-                    if 'extra' in rev:
-                        decrypted_extra = api.decrypt_data(rev['extra'], key)
-                        extra_object = json.loads(decrypted_extra.decode('utf-8'))
-                        extra.update(extra_object)
-                        if 'files' in extra:
-                            for atta in extra_object['files']:
-                                udata['file_ids'].append(atta['id'])
-                                if 'thumbnails' in atta:
-                                    for thumb in atta['thumbnails']:
-                                        udata['file_ids'].append(thumb['id'])
-                    ro['extra'] = api.encrypt_aes(json.dumps(extra).encode('utf-8'), key)
-                    ro['udata'] = udata
-                    rq = {
-                        'command': 'record_update',
-                        'update_records': [ro]
-                    }
-                    rs = api.communicate(params, rq)
-                    if 'update_records' in rs:
-                        params.sync_data = True
-                        if rs['update_records']:
-                            status = rs['update_records'][0]
-                            if status['status'] == 'success':
-                                logging.info('Revision V.{0} is restored'.format(revision))
-                                params.queue_audit_event('revision_restored', record_uid=record_uid)
-                            else:
-                                raise CommandError('history', 'Failed to restore record revision: {0}'.format(status['status']))
-                else:
-                    raise CommandError('history', 'Cannot restore this revision')
-
-    @staticmethod
-    def load_revision(params, record_key, revision):
-        # type: (KeeperParams, bytes, dict) -> Record
-        data = json.loads(api.decrypt_data(revision['data'], record_key).decode('utf-8'))
-        if 'extra' in revision:
-            extra = json.loads(api.decrypt_data(revision['extra'], record_key).decode('utf-8'))
-        else:
-            extra = {}
-        rec = Record(revision['record_uid'])
-        rec.load(data, version=revision.get('version'), extra=extra)
-        return rec
-
-    @staticmethod
-    def get_diff_index(value1, value2):
-        length = min(len(value1), len(value2))
-        for i in range(length):
-            if value1[i] != value2[i]:
-                return i
-        return length
-
-    TargetValueLength = 32
-    @staticmethod
-    def to_diff_str(value, index=0):
-        if len(value) > RecordHistoryCommand.TargetValueLength:
-            if index > 6:
-                tail_len = len(value) - index
-                if tail_len >= RecordHistoryCommand.TargetValueLength - 6:
-                    value = '... ' + value[index-2:]
-                else:
-                    value = '... ' + value[-(RecordHistoryCommand.TargetValueLength - 6):]
-            if len(value) > RecordHistoryCommand.TargetValueLength:
-                value = value[:26] + ' ...'
-        return value
-
-    @staticmethod
-    def compare_values(value1, value2):
-        # type: (str, str) -> (str, str) or None
-        if not value1 and not value2:
-            return None
-        if value1 and value2:
-            if value1 == value2:
-                return None
-            idx = RecordHistoryCommand.get_diff_index(value1, value2)
-            return RecordHistoryCommand.to_diff_str(value1, idx), RecordHistoryCommand.to_diff_str(value2, idx)
-        else:
-            return RecordHistoryCommand.to_diff_str(value1 or ''), RecordHistoryCommand.to_diff_str(value2 or '')
-
-    @staticmethod
-    def to_attachment_str(attachment):
-        # type: (dict) -> str
-        value = ''
-        if attachment:
-            value += attachment.get('title') or attachment.get('name') or attachment.get('id')
-            size = attachment.get('size') or 0
-            scale = 'b'
-            if size > 0:
-                if size > 2000:
-                    size = size / 1024
-                    scale = 'Kb'
-                if size > 2000:
-                    size = size / 1024
-                    scale = 'Mb'
-                if size > 2000:
-                    size = size / 1024
-                    scale = 'Gb'
-                value += ': ' + '{0:.2f}'.format(size).rstrip('0').rstrip('.') + scale
-        return value
-
-    @staticmethod
-    def get_record_diffs(record1, record2):
-        # type: (Record, Record) -> [(str, str, str)]
-        d1 = record1.__dict__
-        d2 = record2.__dict__
-        for field in [('title', 'Title'), ('login', 'Login'), ('password', 'Password'), ('login_url', 'URL'), ('notes', 'Notes'), ('totp', 'Two Factor')]:
-            v1 = d1.get(field[0]) or ''
-            v2 = d2.get(field[0]) or ''
-            cmp_result = RecordHistoryCommand.compare_values(v1, v2)
-            if cmp_result:
-                yield field[1], cmp_result[0], cmp_result[1]
-        if record1.custom_fields or record2.custom_fields:
-            all_keys = set()
-            all_keys.update([x['name'] for x in record1.custom_fields if 'name' in x])
-            all_keys.update([x['name'] for x in record2.custom_fields if 'name' in x])
-            keys = [x for x in all_keys]
-            keys.sort()
-            for key in keys:
-                v1 = record1.get(key) or ''
-                v2 = record2.get(key) or ''
-                cmp_result = RecordHistoryCommand.compare_values(v1, v2)
-                if cmp_result:
-                    yield key[:24], cmp_result[0], cmp_result[1]
-        if record1.attachments or record2.attachments:
-            att1 = {}
-            att2 = {}
-            if record1.attachments:
-                for atta in record1.attachments:
-                    if 'id' in atta:
-                        att1[atta['id']] = atta
-            if record2.attachments:
-                for atta in record2.attachments:
-                    if 'id' in atta:
-                        att2[atta['id']] = atta
-            s = set()
-            s.update(att1.keys())
-            s.symmetric_difference_update(att2.keys())
-            if len(s) > 0:
-                for id in s:
-                    yield 'Attachment', RecordHistoryCommand.to_attachment_str(att1.get(id)), RecordHistoryCommand.to_attachment_str(att2.get(id))
-
-
 class TotpEndpoint:
     def __init__(self, record_uid, record_title, paths):
         self.record_uid = record_uid
@@ -1448,78 +1154,3 @@ class TotpCommand(Command):
                         path = '/' + get_folder_path(params, folder_uid, '/')
                         paths.append(path)
                     TotpCommand.Endpoints.append(TotpEndpoint(record_uid, record.title, paths))
-
-
-class SharedRecordsReport(Command):
-    def get_parser(self):
-        return shared_records_report_parser
-
-    def execute(self, params, **kwargs):
-
-        export_format = kwargs['format'] if 'format' in kwargs else None
-        export_name = kwargs['name'] if 'name' in kwargs else None
-
-        shared_records_data_rs = api.communicate_rest(params, None, 'report/get_shared_record_report', rs_type=SharedRecordResponse)
-
-        shared_from_mapping = {
-            1: "Direct Share",
-            2: "Share Folder",
-            3: "Share Team Folder"
-        }
-
-        rows = []
-        count = 0
-        for e in shared_records_data_rs.events:
-            count = count + 1
-
-            record_uid = api.decode_uid_to_str(e.recordUid)
-
-            cached_record = None
-
-            if record_uid in params.record_cache:   # to avoid not found warning log messages
-                cached_record = api.get_record(params, record_uid)
-
-            if not cached_record:   # probably deleted record
-                logging.debug("Record uid=%s was not located in current cache." % record_uid)
-                continue
-
-            # Folder Path(s)
-            folders = [get_folder_path(params, x) for x in find_folders(params, record_uid)]
-            path_str = ""
-            for i in range(len(folders)):
-                path_str = path_str + ('{0}{1}'.format('\n' if i > 0 else '', folders[i]))
-
-            if not e.canEdit and not e.canReshare:
-                permissions = "Read Only"
-            elif not e.canEdit and e.canReshare:
-                permissions = "Can Share"
-            elif e.canEdit and e.canReshare:
-                permissions = "Can Edit"
-            else:
-                permissions = "Can Edit & Share"
-
-            row = {
-                'count': count,
-                'uid': record_uid,
-                'title': cached_record.title,
-                'shareTo': e.userName,
-                'sharedFrom': shared_from_mapping[e.shareFrom] if e.shareFrom in shared_from_mapping else "Other Share",
-                'permissions': permissions,
-                'folderPath': path_str
-            }
-
-            rows.append(row)
-
-        fields = ['count', 'uid', 'title', 'shareTo', 'sharedFrom', 'permissions', 'folderPath']
-        field_descriptions = fields
-        if export_format == 'table':
-            field_descriptions = ['#', 'Record UID', 'Title', 'Shared To', 'Shared From', 'Permissions', 'Folder Path']
-
-        table = []
-        for raw in rows:
-            row = []
-            for f in fields:
-                row.append(raw[f])
-            table.append(row)
-
-        return dump_report_data(table, field_descriptions, fmt=export_format, filename=export_name)
