@@ -29,8 +29,7 @@ from typing import Optional
 from google.protobuf.json_format import MessageToDict
 from cryptography.hazmat.primitives.asymmetric import ec, rsa
 
-import keepercommander
-from . import aliases, commands, enterprise_commands
+from . import aliases, commands, enterprise_commands, msp_commands
 from .base import raise_parse_exception, suppress_exit, user_choice, Command, dump_report_data
 from .helpers.timeout import (
     enforce_timeout_range, format_timeout, get_delta_from_timeout_setting, get_timeout_setting_from_delta, parse_timeout
@@ -90,11 +89,10 @@ def register_command_info(aliases, command_info):
     aliases['v'] = 'version'
     aliases['sm'] = 'secrets-manager'
     aliases['secrets'] = 'secrets-manager'
-    for p in [whoami_parser, this_device_parser, proxy_parser, login_parser, logout_parser, echo_parser, set_parser, help_parser,
+    for p in [sync_down_parser, whoami_parser, this_device_parser, proxy_parser, login_parser, logout_parser, echo_parser, set_parser, help_parser,
               version_parser, ksm_parser, keepalive_parser, generate_parser, reset_password_parser
               ]:
         command_info[p.prog] = p.description
-    command_info['sync-down|d'] = 'Download & decrypt data'
 
 
 available_ksm_commands = f"""
@@ -152,11 +150,15 @@ Commands to configure and manage the Keeper Secrets Manager platform.
 
 """
 
+sync_down_parser = argparse.ArgumentParser(prog='sync-down|d', description='Download & decrypt data.')
+sync_down_parser.add_argument('-f', '--force', dest='force', action='store_true', help='full data sync')
+
 whoami_parser = argparse.ArgumentParser(prog='whoami', description='Display information about the currently logged in user.')
 whoami_parser.add_argument('-v', '--verbose', dest='verbose', action='store_true', help='verbose output')
 whoami_parser.error = raise_parse_exception
 whoami_parser.exit = suppress_exit
 
+
 this_device_available_command_verbs = ['rename', 'register', 'persistent-login', 'ip-auto-approve', 'timeout']
 this_device_parser = argparse.ArgumentParser(prog='this-device', description='Display and modify settings of the current device.')
 this_device_parser.add_argument('ops', nargs='*', help="operation str: " + ", ".join(this_device_available_command_verbs))
@@ -253,8 +255,9 @@ ksm_parser.exit = suppress_exit
 
 
 version_parser = argparse.ArgumentParser(prog='version|v', description='Displays version of the installed Commander.')
-version_parser.error = raise_parse_exception
 version_parser.add_argument('-v', '--verbose', dest='verbose', action='store_true', help='verbose output')
+version_parser.add_argument('-p', '--packages', action='store_true', help='Show installed Python packages')
+version_parser.error = raise_parse_exception
 version_parser.exit = suppress_exit
 
 
@@ -268,11 +271,11 @@ generate_parser.add_argument('--clipboard', '-cc', dest='clipboard', action='sto
 generate_parser.add_argument('--quiet', '-q', dest='quiet', action='store_true', help='Only print password list')
 generate_parser.add_argument(
     '--password-list', '-p', dest='password_list', action='store_true',
-    help='Also print password list apart from table, csv, json'
+    help='Also print password list apart from formatted table or json'
 )
-generate_parser.add_argument('--output', '-o', dest='output', action='store', help='Output to file')
+generate_parser.add_argument('--output', '-o', dest='output_file', action='store', help='Output to specified file')
 generate_parser.add_argument(
-    '--format', '-f', dest='format', action='store', choices=['table', 'json'],
+    '--format', '-f', dest='output_format', action='store', choices=['table', 'json'],
     default='table', help='Output format for displaying password, strength, and BreachWatch if available'
 )
 generate_parser.add_argument(
@@ -290,21 +293,25 @@ generate_parser.add_argument(
     '--count', '-c', type=int, dest='length', action='store', help='Length of password', default=20
 )
 generate_parser.add_argument(
-    '--symbols', '-s', type=int, dest='symbols', action='store', default=1,
+    '--symbols', '-s', type=int, dest='symbols', action='store',
     help='Minimum number of symbols in password or 0 for none'
 )
 generate_parser.add_argument(
-    '--digits', '-d', type=int, dest='digits', action='store', default=1,
+    '--digits', '-d', type=int, dest='digits', action='store',
     help='Minimum number of digits in password or 0 for none'
 )
 generate_parser.add_argument(
-    '--uppercase', '-u', type=int, dest='uppercase', action='store', default=1,
+    '--uppercase', '-u', type=int, dest='uppercase', action='store',
     help='Minimum number of uppercase letters in password or 0 for none'
 )
 generate_parser.add_argument(
-    '--lowercase', '-l', type=int, dest='lowercase', action='store', default=1,
+    '--lowercase', '-l', type=int, dest='lowercase', action='store',
     help='Minimum number of lowercase letters in password or 0 for none'
 )
+generate_parser.add_argument(
+    '-r', '--rules', dest='rules', action='store',
+    help='Use comma separated complexity integers (uppercase, lowercase, numbers, symbols)'
+)
 generate_parser.error = raise_parse_exception
 generate_parser.exit = suppress_exit
 
@@ -323,8 +330,17 @@ def ms_to_str(ms, frmt='%Y-%m-%d %H:%M:%S'):
 
 
 class SyncDownCommand(Command):
+    def get_parser(self):
+        return sync_down_parser
+
     def execute(self, params, **kwargs):
-        api.sync_down(params)
+        if kwargs.get('force'):
+            params.revision = 0
+            if params.config:
+                if 'skip_records' in params.config:
+                    del params.config['skip_records']
+
+        api.sync_down(params, record_types=True)
 
         accepted = False
         if len(params.pending_share_requests) > 0:
@@ -488,39 +504,52 @@ class ThisDeviceCommand(Command):
     def print_device_info(params: KeeperParams):
         acct_summary_dict, this_device = ThisDeviceCommand.get_account_summary_and_this_device(params)
 
-        print('{:>20}: {}'.format('Device Name', this_device['deviceName']))
-        print("{:>20}: {}".format('Client Version', this_device['clientVersion']))
+        print('{:>32}: {}'.format('Device Name', this_device['deviceName']))
+        # print("{:>32}: {}".format('API Client Version', rest_api.CLIENT_VERSION))
 
         if 'encryptedDataKeyPresent' in this_device:
-            print("{:>20}: {}".format('Data Key Present', (bcolors.OKGREEN + 'YES' + bcolors.ENDC) if this_device['encryptedDataKeyPresent'] else (bcolors.FAIL + 'NO' + bcolors.ENDC)))
+            print("{:>32}: {}".format('Data Key Present', (bcolors.OKGREEN + 'YES' + bcolors.ENDC) if this_device['encryptedDataKeyPresent'] else (bcolors.FAIL + 'NO' + bcolors.ENDC)))
         else:
-            print("{:>20}: {}".format('Data Key Present', (bcolors.FAIL + 'missing' + bcolors.ENDC)))
+            print("{:>32}: {}".format('Data Key Present', (bcolors.FAIL + 'missing' + bcolors.ENDC)))
 
         if 'ipDisableAutoApprove' in acct_summary_dict['settings']:
             ipDisableAutoApprove = acct_summary_dict['settings']['ipDisableAutoApprove']
             # ip_disable_auto_approve - If enabled, the device is NOT automatically approved
             # If disabled, the device will be auto approved
             ipAutoApprove = not ipDisableAutoApprove
-            print("{:>20}: {}".format('IP Auto Approve',
+            print("{:>32}: {}".format('IP Auto Approve',
                                       (bcolors.OKGREEN + 'ON' + bcolors.ENDC)
                                       if ipAutoApprove else
                                       (bcolors.FAIL + 'OFF' + bcolors.ENDC)))
         else:
-            print("{:>20}: {}".format('IP Auto Approve', (bcolors.OKGREEN + 'ON' + bcolors.ENDC)))
+            print("{:>32}: {}".format('IP Auto Approve', (bcolors.OKGREEN + 'ON' + bcolors.ENDC)))
             # ip_disable_auto_approve = 0 / disabled (default) <==> IP Auto Approve :ON
 
         persistentLogin = acct_summary_dict['settings'].get('persistentLogin', False)
-        print("{:>20}: {}".format('Persistent Login',
+        print("{:>32}: {}".format('Persistent Login',
                                   (bcolors.OKGREEN + 'ON' + bcolors.ENDC)
                                   if persistentLogin and not ThisDeviceCommand.is_persistent_login_disabled(params) else
                                   (bcolors.FAIL + 'OFF' + bcolors.ENDC)))
 
         if 'logoutTimer' in acct_summary_dict['settings']:
-            timeout_delta = get_delta_from_timeout_setting(acct_summary_dict['settings']['logoutTimer'])
-            print("{:>20}: {}".format('Logout Timeout', format_timeout(timeout_delta)))
+            device_timeout = get_delta_from_timeout_setting(acct_summary_dict['settings']['logoutTimer'])
+            print("{:>32}: {}".format('Device Logout Timeout', format_timeout(device_timeout)))
 
         else:
-            print("{:>20}: Default".format('Logout Timeout'))
+            device_timeout = timedelta(hour=1)
+            print("{:>32}: Default".format('Logout Timeout'))
+
+        if 'Enforcements' in acct_summary_dict and 'longs' in acct_summary_dict['Enforcements']:
+            logout_timeout = next((x['value'] for x in acct_summary_dict['Enforcements']['longs']
+                                    if x['key'] == 'logout_timer_desktop'), None)
+            if logout_timeout:
+                enterprise_timeout = timedelta(minutes=int(logout_timeout))
+                print("{:>32}: {}".format('Enterprise Logout Timeout', format_timeout(enterprise_timeout)))
+
+                print("{:>32}: {}".format('Effective Logout Timeout',
+                                          format_timeout(min(enterprise_timeout, device_timeout))))
+
+        print('{:>32}: {}'.format('Is SSO User', params.settings['sso_user'] if 'sso_user' in params.settings else False))
 
         print("\nAvailable sub-commands: ", bcolors.OKBLUE + (", ".join(this_device_available_command_verbs)) + bcolors.ENDC)
 
@@ -583,7 +612,7 @@ class WhoamiCommand(Command):
                     storage_usage = (int(storage_bytes_used) * 100 // storage_bytes) if storage_bytes != 0 else 0     # note: int64 in protobuf in python produces string  as opposed to an int or long.
                     print('{0:>20s}: {1:<20s}'.format('Usage', f'{storage_usage}%'))
                     print('{0:>20s}: {1:<20s}'.format('Storage Renewal Date', params.license['storage_expiration_date']))
-                print('{0:>20s}: {1:<20s}'.format('Breach Watch', 'Yes' if params.license.get('breach_watch_enabled') else 'No'))
+                print('{0:>20s}: {1:<20s}'.format('BreachWatch', 'Yes' if params.license.get('breach_watch_enabled') else 'No'))
                 if params.enterprise:
                     print('{0:>20s}: {1:<20s}'.format('Reporting & Alerts', 'Yes' if params.license.get('audit_and_reporting_enabled') else 'No'))
 
@@ -603,15 +632,15 @@ class WhoamiCommand(Command):
 
 class VersionCommand(Command):
     def get_parser(self):
-        return whoami_parser
+        return version_parser
 
     def is_authorised(self):
         return False
 
     def execute(self, params, **kwargs):
-
         version_details = is_up_to_date_version(params)
-        is_verbose = kwargs.get('verbose') or False
+        is_verbose = kwargs.get('verbose', False)
+        show_packages = kwargs.get('packages', False)
 
         this_app_version = __version__
 
@@ -622,14 +651,15 @@ class VersionCommand(Command):
             print('{0}: {1}'.format('Commander Version', this_app_version))
         else:
             print('{0:>20s}: {1}'.format('Commander Version', this_app_version))
+            print("{0:>20s}: {1}".format('API Client Version', rest_api.CLIENT_VERSION))
             print('{0:>20s}: {1}'.format('Python Version', sys.version.replace("\n", "")))
             print('{0:>20s}: {1}'.format('Operating System', loginv3.CommonHelperMethods.get_os() + '(' + platform.release() + ')'))
             print('{0:>20s}: {1}'.format('Working directory', os.getcwd()))
-            print('{0:>20s}: {1}'.format('Package directory', os.path.dirname(keepercommander.__file__)))
+            print('{0:>20s}: {1}'.format('Package directory', os.path.dirname(api.__file__)))
             print('{0:>20s}: {1}'.format('Config. File', params.config_filename))
             print('{0:>20s}: {1}'.format('Executable', sys.executable))
 
-        if logging.getLogger().isEnabledFor(logging.DEBUG):
+        if logging.getLogger().isEnabledFor(logging.DEBUG) or show_packages:
             import pkg_resources
             installed_packages = pkg_resources.working_set
             installed_packages_list = sorted(["%s==%s" % (i.key, i.version) for i in installed_packages])
@@ -826,7 +856,7 @@ class KSMCommand(Command):
         if ksm_obj in ['app', 'apps'] and ksm_action == 'get':
 
             if len(ksm_command) != 3:
-                print(f"{bcolors.WARNING}Application name is required.\n  Example: {bcolors.OKGREEN}secrets-manager get app {bcolors.OKBLUE}MyApp{bcolors.ENDC}")
+                print(f"{bcolors.WARNING}Application name is required.\n  Example: {bcolors.OKGREEN}secrets-manager app get {bcolors.OKBLUE}MyApp{bcolors.ENDC}")
                 return
 
             ksm_app_uid_or_name = ksm_command[2]
@@ -1127,8 +1157,10 @@ class KSMCommand(Command):
                                 editable_status]
                         elif sht == 'SHARE_TYPE_FOLDER':
                             if uid_str not in params.shared_folder_cache:
-                                logging.warning("Shared folder '%s' is not in the cache. Try to perform the `sync-down` "
-                                                "to download latest data to local cache." % sht)
+                                # logging.warning(f"Shared folder uid {uid_str} is not present in the cache. Looks like "
+                                #                 f"it was removed or was not sync to this client yet. Try to perform "
+                                #                 f"the `sync-down` to download latest data to local cache." % sht)
+                                continue
                             cached_sf = params.shared_folder_cache[uid_str]
                             shf_name = cached_sf.get('name_unencrypted')
                             # shf_num_of_records = len(cached_sf.get('records'))
@@ -1746,6 +1778,8 @@ class HelpCommand(Command):
                 parser = commands[cmd].get_parser()
             elif cmd in enterprise_commands:
                 parser = enterprise_commands[cmd].get_parser()
+            elif cmd in msp_commands:
+                parser = msp_commands[cmd].get_parser()
             if parser:
                 parser.print_help()
 
@@ -1787,8 +1821,9 @@ class GenerateCommand(Command):
         return generate_parser
 
     def execute(self, params, number=None, no_breachwatch=None,
-                length=None, symbols=None, digits=None, uppercase=None, lowercase=None,
-                format=None, output=None, json_indent=None, **kwargs):
+                length=None, symbols=None, digits=None, uppercase=None, lowercase=None, rules=None,
+                output_format=None, output_file=None, json_indent=None, quiet=False, password_list=False,
+                clipboard=False, return_result=False, **kwargs):
         """
         Executes "generate" command
 
@@ -1808,19 +1843,39 @@ class GenerateCommand(Command):
             Minimum number of uppercase letters in password or 0 for none. Default: 1
         lowercase : int
             Minimum number of lowercase letters in password or 0 for none. Default: 1
+        rules : str
+            Use comma separated complexity integers (uppercase, lowercase, numbers, symbols)
         no_breachwatch : bool
             Skip BreachWatch detection if BreachWatch is enabled for this account
-        format : str
+        output_format : str
             Output format for displaying password, strength, and BreachWatch if available. 'table' or 'json'
-        output : str
+        output_file : str
             File name to store result. stdout is omitted
         json_indent : int
             JSON format indent (0 for compact, >0 for pretty print). Default: 2
+        quiet : bool
+            Only print password list
+        password_list: bool
+            Also print password list apart from formatted table or json
+        clipboard: bool
+            Copy to clipboard
+        return_result : bool
+            If True return tuple of password dict and formatted output string
         """
 
-        kpg = KeeperPasswordGenerator(
-            length, symbols, digits, uppercase, lowercase
-        )
+        if rules and all(i is None for i in (symbols, digits, uppercase, lowercase)):
+            kpg = KeeperPasswordGenerator.create_from_rules(rules, length)
+            if kpg is None:
+                logging.warning('Using default password complexity rules')
+                kpg = KeeperPasswordGenerator(length=length)
+        else:
+            if rules:
+                logging.warning(
+                    'Ignoring "rules" option used with "symbols", "digits", "uppercase", or "lowercase" option'
+                )
+            kpg = KeeperPasswordGenerator(
+                length=length, symbols=symbols, digits=digits, caps=uppercase, lower=lowercase
+            )
         get_new_password_count = number
         no_breachwatch = no_breachwatch or getattr(params, 'breach_watch', None) is None
 
@@ -1850,39 +1905,43 @@ class GenerateCommand(Command):
                         )
                 params.breach_watch.delete_euids(params, euids)
 
-        if kwargs['quiet']:
-            format_output = ''
+        if quiet:
+            formatted_output = ''
+        elif output_format == 'table':
+            breach_watch = '' if no_breachwatch else '{breach_watch:13}'
+            format_template = '{count:<5}{strength:<13}' + breach_watch + '{password}'
+            header = format_template.format(
+                count='', strength='Strength(%)', breach_watch='BreachWatch', password='Password'
+            )
+            password_output = [format_template.format(count=i, **p) for i, p in enumerate(passwords, start=1)]
+            formatted_output = header + '\n' + '\n'.join(password_output)
+        elif output_format == 'json':
+            formatted_output = json.dumps(passwords, indent=json_indent or None)
         else:
-            if format == 'table':
-                breach_watch = '' if no_breachwatch else '{breach_watch:13}'
-                format_template = '{count:<5}{strength:<13}' + breach_watch + '{password}'
-                header = format_template.format(
-                    count='', strength='Strength(%)', breach_watch='BreachWatch', password='Password'
-                )
-                password_output = [format_template.format(count=i + 1, **p) for i, p in enumerate(passwords)]
-                format_output = header + '\n' + '\n'.join(password_output)
-            elif format == 'json':
-                format_output = json.dumps(passwords, indent=json_indent or None)
+            formatted_output = ''
 
-        if kwargs['quiet'] or kwargs['password_list']:
-            skip_line = '\n\n' if kwargs['password_list'] else ''
-            format_output += skip_line + '\n'.join(p['password'] for p in passwords)
+        if quiet or password_list:
+            skip_line = '\n\n' if password_list else ''
+            formatted_output += skip_line + '\n'.join(p['password'] for p in passwords)
 
-        if kwargs['clipboard']:
+        if clipboard:
             import pyperclip
-            pyperclip.copy(format_output)
+            pyperclip.copy(formatted_output)
             logging.info('New passwords copied to clipboard')
-        elif not output:
-            print(format_output)
+        elif not output_file:
+            print(formatted_output)
 
-        if output:
+        if output_file:
             try:
-                with open(output, 'w') as f:
-                    f.write(format_output)
+                with open(output_file, 'w') as f:
+                    f.write(formatted_output)
             except Exception as e:
-                logging.warning('Error writing to file {}: {}'.format(output, str(e)))
+                logging.warning('Error writing to file {}: {}'.format(output_file, str(e)))
             else:
-                logging.info('Wrote to file {}'.format(output))
+                logging.info('Wrote to file {}'.format(output_file))
+
+        if return_result:
+            return passwords, formatted_output
 
 
 class ResetPasswordCommand(Command):
