@@ -38,10 +38,9 @@ class bcolors:
 def welcome():
     lines = []    # type: List[Union[str, Tuple[str, str]]]
 
-    lines.append( r'           ___________    /#. ')
-    lines.append( r'         /###########/   /###\ ')
-    lines.append( r'        /###########/   /#####\      _    __  _______  _______  ______   _______  ______ (R)')
-    lines.append( r'       /###########/    \######\    | |  / / |  ____/ |  ____/ |  ___ \ |  ____/ |  ___ \ ')
+    lines.append( r'         /#############/   /#\ ')
+    lines.append( r'        /#############/   /###\      _    __  _______  _______  ______   _______  ______ (R)')
+    lines.append( r'       /#############/   /#####\    | |  / / |  ____/ |  ____/ |  ___ \ |  ____/ |  ___ \ ')
     lines.append( r'      /######/           \######\   | | / /  | |____  | |____  | | __| || |____  | | __| | ')
     lines.append( r'     /######/             \######\  | |< <   |  ___/  |  ___/  | |/___/ |  ___/  | |/_  / ')
     lines.append( r'    /######/               \######\ | | \ \  | |_____ | |_____ | |      | |_____ | |  \ \ ')
@@ -50,7 +49,7 @@ def welcome():
     lines.append((r'      \######\           /######/ ', r'   /  ___|___  _ __ ___  _ __ ___   __ _ _ __   __| | ___ _ __ '))
     lines.append((r'       \#############\   \#####/  ', r"  /  /   / _ \| '_ ` _ \| '_ ` _ \ / _` | '_ \ / _` |/ _ \ '__| "))
     lines.append((r'        \#############\   \###/   ', r'  \  \__| (_) | | | | | | | | | | | (_| | | | | (_| |  __/ | '))
-    lines.append((r'         \#############\   \#.    ', r'   \_____\___/|_| |_| |_|_| |_| |_|\__,_|_| |_|\__,_|\___|_| '))
+    lines.append((r'         \#############\   \#/    ', r'   \_____\___/|_| |_| |_|_| |_| |_|\__,_|_| |_|\__,_|\___|_| '))
     lines.append('')
 
     try:
