@@ -11,11 +11,13 @@ import logging
 import os
 import random
 import string
+from typing import Optional
 from secrets import choice
 
 from Cryptodome.Random.random import shuffle
 
 
+DEFAULT_PASSWORD_LENGTH = 32
 PW_SPECIAL_CHARACTERS = '!@#$%()+;<>=?[]{}^.,'
 
 
@@ -91,15 +93,22 @@ def generate(length=64):
 
 
 class KeeperPasswordGenerator:
-    def __init__(self, length: int, symbols: int, digits: int, caps: int, lower: int):
-        sum_categories = sum(
-            (symbols if symbols > 0 else 0, digits if digits > 0 else 0, caps if caps > 0 else 0, lower if lower > 0 else 0)
-        )
-        if sum_categories == 0:
+    def __init__(self, length: int = DEFAULT_PASSWORD_LENGTH, symbols: Optional[int] = None,
+                 digits: Optional[int] = None, caps: Optional[int] = None, lower: Optional[int] = None,
+                 special_characters: str = PW_SPECIAL_CHARACTERS):
+        none_count = (symbols, digits, caps, lower).count(None)
+        sum_categories = sum(0 if i is None or i <= 0 else i for i in (symbols, digits, caps, lower))
+        if none_count > 0:
+            # remaining length of password will be divided among unspecified categories
+            extra_count = length - sum_categories if length > sum_categories else 0
+            new_none_value = extra_count // none_count
+            symbols, digits, caps, lower = (new_none_value if i is None else i for i in (symbols, digits, caps, lower))
+            sum_categories += new_none_value * none_count
+        elif sum_categories == 0:
             symbols, digits, caps, lower, sum_categories = 1, 1, 1, 1, 4
         extra_count = length - sum_categories if length > sum_categories else 0
         self.category_map = [
-            (symbols, PW_SPECIAL_CHARACTERS),
+            (symbols, special_characters),
             (digits, string.digits),
             (caps, string.ascii_uppercase),
             (lower, string.ascii_lowercase),
@@ -113,3 +122,27 @@ class KeeperPasswordGenerator:
             password_list.extend(choice(chars) for i in range(count))
         shuffle(password_list)
         return ''.join(password_list)
+
+    @classmethod
+    def create_from_rules(cls, rule_string: str, length: Optional[int] = None,
+                          special_characters: str = PW_SPECIAL_CHARACTERS):
+        """Create instance of class from rules string
+
+        rule_string: comma separated integer character counts of uppercase, lowercase, numbers, symbols
+        length: length of password
+        special_characters: set of characters used to generate password symbols
+        """
+        rule_list = [s.strip() for s in rule_string.split(',')]
+        if len(rule_list) != 4 or not all(n.isnumeric() for n in rule_list):
+            logging.warning(
+                'Invalid rules to generate password. Format is "upper, lower, digits, symbols"'
+            )
+            return None
+        else:
+            rule_list = [int(n) for n in rule_list]
+            upper, lower, digits, symbols = rule_list
+            length = sum(rule_list) if length is None else length
+            return cls(
+                length=length, caps=upper, lower=lower, digits=digits, symbols=symbols,
+                special_characters=special_characters
+            )
