@@ -45,7 +45,8 @@ import_parser.add_argument('--display-csv', '-dc', dest='display_csv', action='s
 import_parser.add_argument('--display-json', '-dj', dest='display_json', action='store_true',
                            help='display Keeper JSON import instructions')
 import_parser.add_argument(
-    '--format', dest='format', choices=['json', 'csv', 'keepass', 'lastpass', 'myki'], required=True, help='file format'
+    '--format', choices=['json', 'csv', 'keepass', 'lastpass', 'myki', 'manageengine', '1password'],
+    required=True, help='file format'
 )
 import_parser.add_argument('--folder', dest='folder', action='store',
                            help='import into a separate folder.')
@@ -67,7 +68,9 @@ import_parser.add_argument('--new-domain', '-nd', dest='new_domain', action='sto
                            help='new domain for changing user emails in permissions')
 import_parser.add_argument('--file-cache', dest='tmpdir', action='store',
                            help='Temp directory used to cache encrypted attachment imports')
-import_parser.add_argument('name', type=str, help='file name (json, csv, keepass) or account name (lastpass)')
+import_parser.add_argument(
+    'name', type=str, help='file name (json, csv, keepass, 1password), account name (lastpass), or URL (ManageEngine)'
+)
 import_parser.error = raise_parse_exception
 import_parser.exit = suppress_exit
 
@@ -77,6 +80,7 @@ export_parser.add_argument('--format', dest='format', choices=['json', 'csv', 'k
 export_parser.add_argument('--max-size', dest='max_size', help='Maximum file attachment file. Example: 100K, 50M, 2G. Default: 10M')
 export_parser.add_argument('-kp', '--keepass-file-password', dest='keepass_file_password', action='store', help='Password for the exported Keepass file')
 export_parser.add_argument('--force', dest='force', action='store_true', help='Suppress user interaction. Assume "yes"')
+export_parser.add_argument('--folder', dest='folder', action='store', help='Export data from the specific folder only.')
 export_parser.add_argument('name', type=str, nargs='?', help='file name or console output if omitted (except keepass)')
 export_parser.error = raise_parse_exception
 export_parser.exit = suppress_exit
