@@ -9,41 +9,39 @@
 # Contact: ops@keepersecurity.com
 #
 
+from typing import Iterator, List, Optional, Union, Dict, Tuple, Set
+
 """Import and export functionality."""
 import abc
 import base64
 import collections
 import copy
 import hashlib
-import itertools
 import json
 import logging
 import os
 import re
-import time
-from typing import Iterator, List, Optional, Union, Dict, Tuple
-from contextlib import contextmanager
-
+import itertools
 import math
 import requests
-from Cryptodome.Cipher import AES
+import time
 
-from keepercommander import api
-from keepercommander.display import bcolors
-from keepercommander.rest_api import CLIENT_VERSION  # pylint: disable=no-name-in-module
 from .encryption_reader import EncryptionReader
 from .importer import (importer_for_format, exporter_for_format, path_components, PathDelimiter, BaseExporter,
                        BaseImporter, Record as ImportRecord, RecordField as ImportRecordField, Folder as ImportFolder,
                        SharedFolder as ImportSharedFolder, Permission as ImportPermission, BytesAttachment,
                        Attachment as ImportAttachment, RecordSchemaField, File as ImportFile,
                        RecordReferences, FIELD_TYPE_ONE_TIME_CODE)
+from .. import api
 from .. import utils, crypto
 from ..commands import base
-from ..proto import record_pb2, folder_pb2
+from ..display import bcolors
 from ..error import KeeperApiError, CommandError
 from ..params import KeeperParams
+from ..proto import record_pb2, folder_pb2
 from ..recordv3 import RecordV3
-from ..subfolder import BaseFolderNode, SharedFolderFolderNode, find_folders
+from ..rest_api import CLIENT_VERSION  # pylint: disable=no-name-in-module
+from ..subfolder import BaseFolderNode, SharedFolderFolderNode, find_folders, try_resolve_path
 
 EMAIL_PATTERN = r"(^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$)"
 IV_LEN = 12
@@ -62,66 +60,6 @@ def get_record_data_json_bytes(data):
         data_str = data_str.ljust(padding)
     return data_str.encode('utf-8')
 
-"""
-def exceed_max_data_len(data, import_record, map_data_custom_to_rec_fields):
-    data_json = api.get_record_data_json_bytes(data)
-    data_size = len(data_json) + 28
-    if data_size > RECORD_MAX_DATA_LEN:
-        field_sizes = [len(f['value'][0]) if len(f['value']) > 0 else 0 for f in data['fields']]
-        custom_sizes = [len(f['value'][0]) if len(f['value']) > 0 else 0 for f in data['custom']]
-        note_size = len(data['notes'])
-        attachment_filenames = [a.name for a in import_record.attachments or []]
-        while data_size > RECORD_MAX_DATA_LEN:
-            max_field_size = max(field_sizes) if field_sizes else 0
-            max_custom_size = max(custom_sizes) if custom_sizes else 0
-            max_size = max(note_size, max_field_size, max_custom_size)
-            title = data['title']
-            if max_size == 0:
-                logging.warning(RECORD_MAX_DATA_WARN.format(title, data_size, RECORD_MAX_DATA_LEN))
-                return True
-
-            atta = ImportAttachment()
-            if note_size == max_size:
-                atta.name = f'{title}_notes_field.txt'
-                atta_data = data['notes'].encode('utf-8')
-                data['notes'] = LARGE_FIELD_MSG.format(atta.name)
-                import_record.notes = data['notes']
-                note_size = 0
-            elif max_custom_size == max_size:
-                custom_index = custom_sizes.index(max_size)
-                field = data['custom'][custom_index]
-                field_name = field.get('label', field.get('type', 'custom'))
-                atta.name = f'{title}_{field_name}_field.txt'
-                atta_data = field['value'][0].encode('utf-8')
-                field['value'][0] = LARGE_FIELD_MSG.format(atta.name)
-                import_record_field = import_record.fields[map_data_custom_to_rec_fields[custom_index]]
-                import_record_field.value = field['value'][0]
-                custom_sizes[custom_index] = 0
-            else:
-                logging.warning(RECORD_MAX_DATA_WARN.format(title, data_size, RECORD_MAX_DATA_LEN))
-                return True
-
-            def atta_open():
-                return io.BytesIO(atta_data)
-
-            atta.open = atta_open
-            atta.size = len(atta_data)
-
-            i = 1
-            original_name = atta.name[:-4]
-            while atta.name in attachment_filenames:
-                atta.name = f'{original_name}({i}).txt'
-                i += 1
-            attachment_filenames.append(atta.name)
-
-            if import_record.attachments:
-                import_record.attachments.append(atta)
-            else:
-                import_record.attachments = [atta]
-            data_json = api.get_record_data_json_bytes(data)
-            data_size = len(data_json) + 28
-    return False
-"""
 
 def get_import_folder(params, folder_uid, record_uid):
     """Get a folder name from a folder uid (?)."""
@@ -247,7 +185,7 @@ def convert_keeper_record(record, has_attachments=False):
                 rec.login = field_value
             elif field_type == 'password' and not rec.password and type(field_value) == str:
                 rec.password = field_value
-            elif field_type == 'url' and not rec.login_url and type(field_value) == str:
+            elif field_type == 'url' and not field.get('label') and not rec.login_url and type(field_value) == str:
                 rec.login_url = field_value
             elif field_type.endswith('Ref'):
                 ref_type = field_type[:-3]
@@ -277,6 +215,7 @@ def convert_keeper_record(record, has_attachments=False):
 
 
 def export(params, file_format, filename, **kwargs):
+    # type: (KeeperParams, str, str, ...) -> None
     """Export data from Vault to a file in an assortment of formats."""
     api.sync_down(params)
 
@@ -284,11 +223,37 @@ def export(params, file_format, filename, **kwargs):
     if 'max_size' in kwargs:
         exporter.max_size = int(kwargs['max_size'])
 
+    folder_filter = None      # type: Optional[Set[str]]
+    record_filter = None      # type: Optional[Set[str]]
+    folder_path = kwargs.get('folder')
+    if folder_path:
+        folder = None    # type: Optional[BaseFolderNode]
+        rs = try_resolve_path(params, folder_path)
+        if rs:
+            f, rest = rs
+            if not rest:
+                folder = f
+        if not folder:
+            logging.warning('Folder \"%s\" not found', folder_path)
+            return
+        folder_filter = set()
+        record_filter = set()
+
+        def on_folder(base_folder):   # type: (BaseFolderNode) -> None
+            folder_filter.add(base_folder.uid)
+            if base_folder.uid in params.subfolder_record_cache:
+                record_filter.update(params.subfolder_record_cache[base_folder.uid])
+        base.FolderMixin.traverse_folder_tree(params, folder.uid, on_folder)
+
     to_export = []
     if exporter.has_shared_folders():
         shfolders = [api.get_shared_folder(params, sf_uid) for sf_uid in params.shared_folder_cache]
         shfolders.sort(key=lambda x: x.name.lower(), reverse=False)
         for f in shfolders:
+            if folder_filter:
+                if f.shared_folder_uid not in folder_filter:
+                    continue
+
             fol = ImportSharedFolder()
             fol.uid = f.shared_folder_uid
             fol.path = get_folder_path(params, f.shared_folder_uid)
@@ -316,9 +281,8 @@ def export(params, file_format, filename, **kwargs):
             to_export.append(fol)
     sf_count = len(to_export)
 
-    v3_enabled = params.settings.get('record_types_enabled') if params.settings else False
     force = kwargs.get('force', False)
-    if not force and v3_enabled and not exporter.supports_v3_record():
+    if not force and not exporter.supports_v3_record():
         answer = base.user_choice(f'Export to {file_format} does not fully support typed records\n\n'
                                   'Do you want to continue?', 'yn', 'n')
         if answer.lower() != 'y':
@@ -332,6 +296,10 @@ def export(params, file_format, filename, **kwargs):
     #     external_ids[record_uid] = ext_id
 
     for record_uid in params.record_cache:
+        if record_filter:
+            if record_uid not in record_filter:
+                continue
+
         record = params.record_cache[record_uid]
         record_version = record.get('version') or 0
         if record_version == 2 or record_version == 3:
@@ -380,11 +348,14 @@ def export(params, file_format, filename, **kwargs):
                                         rec.attachments.append(atta)
 
             for folder_uid in find_folders(params, record_uid):
+                if folder_filter:
+                    if folder_uid not in folder_filter:
+                        continue
                 if folder_uid in params.folder_cache:
-                    folder = get_import_folder(params, folder_uid, record_uid)
+                    export_folder = get_import_folder(params, folder_uid, record_uid)
                     if rec.folders is None:
                         rec.folders = []
-                    rec.folders.append(folder)
+                    rec.folders.append(export_folder)
 
             to_export.append(rec)
         # elif record_version == 4:
@@ -469,8 +440,7 @@ def _import(params, file_format, filename, **kwargs):
     old_domain = kwargs.get('old_domain')
     new_domain = kwargs.get('new_domain')
     tmpdir = kwargs.get('tmpdir')
-    v3_enabled = params.settings.get('record_types_enabled', False) if params.settings else False
-    record_type = kwargs.get('record_type') if v3_enabled else None
+    record_type = kwargs.get('record_type')
 
     import_into = kwargs.get('import_into') or ''
     if import_into:
@@ -485,8 +455,8 @@ def _import(params, file_format, filename, **kwargs):
     records = []        # type: List[ImportRecord]
     files = []          # type: List[ImportFile]
 
-    for x in importer.execute(filename, users_only=import_users, old_domain=old_domain, new_domain=new_domain,
-                              tmpdir=tmpdir):
+    for x in importer.execute(filename, params=params, users_only=import_users,
+                              old_domain=old_domain, new_domain=new_domain, tmpdir=tmpdir):
         if isinstance(x, ImportRecord):
             if shared or import_into:
                 if not x.folders:
@@ -567,15 +537,13 @@ def _import(params, file_format, filename, **kwargs):
     audit_uids = []
 
     if records:  # create/update records
-        v3_enabled = params.settings.get('record_types_enabled') if params.settings else False
-
         records_v2_to_add = []      # type: List[folder_pb2.RecordRequest]
         records_v2_to_update = []   # type: List[dict]
         records_v3_to_add = []      # type: List[record_pb2.RecordAdd]
         records_v3_to_update = []   # type: List[record_pb2.RecordUpdate]
         import_uids = {}
 
-        records_to_import, external_lookup = prepare_record_add_or_update(update_flag, params, records, v3_enabled)
+        records_to_import, external_lookup = prepare_record_add_or_update(update_flag, params, records)
 
         reference_uids = set()
         for import_record in records_to_import:
@@ -604,8 +572,6 @@ def _import(params, file_format, filename, **kwargs):
             if existing_record:
                 version = existing_record.get('version', 0)
                 if version == 3:   # V3
-                    if not v3_enabled:
-                        continue
                     orig_data = json.loads(existing_record['data_unencrypted'])
                     if not import_record.type:
                         import_record.type = orig_data.get('type', 'login')
@@ -678,7 +644,7 @@ def _import(params, file_format, filename, **kwargs):
                         if folder.type == BaseFolderNode.RootFolderType:
                             folder_uid = ''
 
-                if import_record.type and v3_enabled:   # V3
+                if import_record.type:   # V3
                     v3_add_rq = record_pb2.RecordAdd()
                     v3_add_rq.record_uid = utils.base64_url_decode(import_record.uid)
                     import_uids[import_record.uid] = {'ver': 'v3', 'op': 'add'}
@@ -839,7 +805,7 @@ def _import(params, file_format, filename, **kwargs):
 
                     if missing_attachments:
                         r.uid = external_lookup[r.uid]
-                        if v3_enabled and r.type:
+                        if r.type:
                             v3_atts.append(r)
 
         if len(v2_atts) > 0:
@@ -1623,8 +1589,8 @@ def build_record_hash(tokens):    # type: (Iterator[str]) -> str
     return hasher.hexdigest()
 
 
-def prepare_record_add_or_update(update_flag, params, records, v3_enabled):
-    # type: (bool, KeeperParams, Iterator[ImportRecord], bool) -> Tuple[List[ImportRecord], dict]
+def prepare_record_add_or_update(update_flag, params, records):
+    # type: (bool, KeeperParams, Iterator[ImportRecord]) -> Tuple[List[ImportRecord], dict]
     """
     Find what records to import or update.
 
@@ -1652,7 +1618,7 @@ def prepare_record_add_or_update(update_flag, params, records, v3_enabled):
     external_lookup = {}
 
     for import_record in records:
-        if v3_enabled and import_record.type:
+        if import_record.type:
             if len(import_record.notes or '') > RECORD_MAX_DATA_LEN - 2 * (2 ** 10):
                 if import_record.attachments is None:
                     import_record.attachments = []
