@@ -148,6 +148,7 @@ class KeeperParams:
         self.breach_watch_records = None
         self.sso_login_info = None
         self.__proxy = None
+        self.ssh_agent = None
 
     def clear_session(self):
         self.auth_verifier = None
@@ -203,6 +204,9 @@ class KeeperParams:
         self.breach_watch = None
         self.breach_watch_records = None
         self.sso_login_info = None
+        if self.ssh_agent:
+            self.ssh_agent.close()
+            self.ssh_agent = None
 
     def __get_rest_context(self):   # type: () -> RestApiContext
         return self.__rest_context
