@@ -123,7 +123,8 @@ def path_split(params, folder, path_string):
     return folder, components
 
 
-def try_resolve_path(params, path):     # type: (KeeperParams, str) -> Tuple[BaseFolderNode, Optional[str]]
+def try_resolve_path(params, path):
+    # type: (KeeperParams, str) -> Tuple[BaseFolderNode, Optional[str]]
     """
     Look up the final keepercommander.subfolder.UserFolderNode and name of the final component(s).
 
@@ -200,7 +201,6 @@ class BaseFolderNode:
             print('{0:>20s}: {1:<20s}'.format('Shared Folder UID', self.shared_folder_uid))
 
 
-
 class UserFolderNode(BaseFolderNode):
     def __init__(self):
         BaseFolderNode.__init__(self, BaseFolderNode.UserFolderType)
